/*
Required input files:	
	Population by year and age-gender.xlsx
	meps_hh_2000_2003

Output files:
	Table_2.csv
	Table_3.csv
	Table_4.csv
*/

*********************
* Initialization
*********************

* Set this local equal to the folder location for this script, e.g., "C:/Users/name/myfolder/"
local workdir ""

version 14
clear
set more off
set varabbrev off
tempfile pops

* These modules contain mata code to calculate conventional value and insurance value
run "`workdir'modules.do"

**********************
** Grab death rate data and convert to survival rates
**********************
local run_no = 1 
foreach sex in Male Female {
	import excel using "`workdir'data/Population by year and age-gender.xlsx", clear sheet("`sex' life tables") firstrow
	keep if Year=="2000-2009"
	replace Age = "110" if Age=="110+"
	destring Age, replace

	gen surv = 1-qx in 1
	replace surv = surv[_n-1]*(1-qx) if _n>1	
	gen tmp = surv if Age==17
	egen normal = mean(tmp)	
	replace surv = surv/normal
	
	drop if Age<18
	ren Age age_bin
	
	* These data ends at 110, so we cannot calculate surv for the endpoint of agebin 80. Set surv=1; that estimate is high, but approximately offset that below by assuming only max of 10 years spent in this bin.
	keep if inlist(age_bin, 18, 35, 50, 65, 80)
	replace surv = 1 if age_bin==80
	
	keep age_bin surv
	gen sex=`run_no'
	if `run_no'==2 append using "`pops'"
	save "`pops'", replace
	local run_no = 2
}

**********************
** Grab EQ-5D data from MEPS 2000-2003 household file
**********************
use "`workdir'data/meps_hh_2000_2003.dta", clear

table year, c(sum perwt) format(%19.0fc)
table year if !mi(healthwt), c(sum perwt) format(%19.0fc)

* Replace negative weights (about 2% of sample) with 0.01 to meet assumption that health quality is in interval (0,1]
drop if mi(healthwt)
replace healthwt = .01 if healthwt<0

assert !mi(age)
replace age = 85 if age>85

egen age_bin = cut(age), at(18 35 50 65 80 120)

* Calculate health quantiles, by age and gender
local tmp ""
local num_groups = 0
foreach x in 10 20 30 40 50 60 70 80 90 {
	local tmp "`tmp' (p`x')healthwtp`x'=healthwt"
	local num_groups = `num_groups'+1
}
gen count = 1
collapse `tmp' (rawsum) count [pw=perwt], by(age_bin sex year) fast
collapse  (sum) count (mean) health*, by(age_bin sex) fast
sort sex age_bin
format healthwt* %5.3gc

di `num_groups'
outsheet sex age_bin count health* using "`workdir'results/Table_2.csv", comma replace

***************
** Set income and health improvement parameters
***************
* Rescale the health weights so they are consistent with consumption (max=$120,000)
local scaling = 120000
foreach v of varlist healthwt* {
	replace `v' = `v'*`scaling'
}

* Set income in well state equal to $120,000, and assume income is sick state is 20% lower. Assume health increase is 5%. Assume equal-weighted probabilities in each health state. Initialize price to 0.
gen y_w = `scaling'
forval x = 1/`=`num_groups'-1' {
	gen prob`x' = 1/`num_groups'
	gen y_s`x' = y_w*0.8
	gen Price`x' = 0
	
	local ptile = `x'*10
	ren healthwtp`ptile' h_s`x'
	gen h_Delta`x' = min(h_s`x' + 0.05*`scaling',1*`scaling')
	gen Delta`x' = h_Delta`x'-h_s`x'
}
ren healthwtp90 h_w
assert abs(h_w-1*`scaling')<.0001

******
* Constrain the health gains to be in the bottom twenty percent
******
egen Delta_total = rsum(Delta*)
forval x = 1/`=`num_groups'-1' {
	gen old_h_Delta`x' = h_Delta`x'
	gen old_Delta`x' = Delta`x'
	
	replace h_Delta`x' = h_s`x'
	replace Delta`x' = 0
}
replace h_Delta1 = min(h_s1+Delta_total/2,1*`scaling')
replace h_Delta2 = min(h_s2+Delta_total-(h_Delta1-h_s1),1*`scaling')
replace Delta1 = h_Delta1-h_s1
replace Delta2 = h_Delta2-h_s2
assert h_Delta1<=1*`scaling' & h_Delta2<=1*`scaling'

**********************
** Estimate insurance value, first with price=0
**********************

* The mata function -estimate_wtp- calculates the traditional/conventional value (CV), the total insurance value (IV), and the financial spending risk (FIV)
* Note: price is 0, so FIV will also be 0
format h_* %12.0fc
gen sigma = 2
mata: estimate_wtp(10000,"CV_nop","IV_nop","FIV_nop")
assert abs(FIV_nop)<0.000001

* Insurance value (IV) = Physical insurance value (PIV) - Financial spending risk (FIV) --> PIV = IV+FIV
gen PIV_nop = IV_nop+FIV_nop

* Now, assume price equal to 25% of conventional value in each health state, which will result in positive financial spending risk (FIV)
forval x = 1/`=`num_groups'-1' {
	replace Price`x' = (_CV_nop`x'*0.25)/prob`x'
}
drop _*
mata: estimate_wtp(10000,"CV","IV","FIV")
drop _*
gen PIV = IV+FIV
assert FIV>0

* Outsheet age-specific results
gen Total = CV + PIV - FIV
sort sex age_bin
keep  sex age_bin CV PIV FIV Total
order sex age_bin CV PIV FIV Total
outsheet using "`workdir'results/Table_3.csv", comma replace

**********************
** Aggregate across ages
**********************
bysort sex (age_bin): gen num_years = age_bin[_n+1]-age_bin
replace num_years = 10 if age_bin==80

merge 1:1 sex age_bin using "`pops'", assert(match) nogenerate
sort sex age_bin

* Discount values by midpoint of age bin
by sex: gen rsum = sum(num_years)
gen discount_yrs = rsum - num_years/2
gen discount = 1 / ((1+0.03)^(discount_yrs))
drop rsum discount_yrs

foreach v of varlist CV PIV FIV {
	gen `v'_total = `v'*num_years*surv*discount
}

collapse (sum) *_total, by(sex) fast
ren *_total *
order sex CV PIV FIV  
gen Total = CV + PIV - FIV
gen Value_added = (PIV-FIV)/CV
outsheet using "`workdir'results/Table_4.csv", comma replace

** EOF
